-- read_in.vhd
-- jamieson olsen <jamieson@fnal.gov>

-- *** revision history ***

-- 25 Jun 1999: original design.  colored busses, left and right handed versions.
-- 11 Dec 2001:	sector neutral design.  numbered busses.
-- 07 Jan 2002:	use LINK_DSKEW modules to deskew and flatten input links.
-- 25 Feb 2002: include Synplify compiler directives.

-- what it duz:

-- six 28-bit wide buses bring CFT/CPS singlets and control bits into 
-- the fpga in seven word frames.  The singlets are separated and formed into 
-- doublets, and then passed onto to the track equation logic.

-- The DOUBLETS component referenced here CHANGES depending on which sector is 
-- the home sector. There are 16 unique sectors in a super-sector; thus there are 
-- 16 unique doublet files.  The doublet file will takes into account the bus number
-- mapping that changes in odd/even sectors.  

-- For example, if the home sector is EVEN this MAXIMUM design will go into the bottom
-- DFEA daughterboard.  In this case, LINK0 will be the BLUE link from the HOME sector.
-- However, if the home sector is ODD, this design will go into the top daughterboard,
-- and LINK0 will be the RED link from the PREV sector.
-- run2b modification:
-- 
library ieee,mylib;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use mylib.dfea_pack.all;

-- library synplify;
-- use synplify.attributes.all;

entity read_in is
 port (
		version:		in 	std_logic_vector (15 downto 0);
		link1: 		in 	std_logic_vector (27 downto 0);
		link1_clk:	in 	std_logic;
		link2: 		in 	std_logic_vector (27 downto 0);
		link2_clk:	in 	std_logic;
		link3: 		in 	std_logic_vector (27 downto 0);
		link3_clk:	in 	std_logic;
		link4: 		in 	std_logic_vector (27 downto 0);
		link4_clk:	in 	std_logic;

		clock: 		in 	std_logic;	-- the board clock
		reset:		out		std_logic;	-- active low
		clear:		in		std_logic;	-- reset the read pointers
		u9:			in		std_logic;	
		eqs_vers:	in	std_logic_vector(31 downto 0);	-- equation version and sector
		veto_vers:	in 	std_logic_vector (15 downto 0);
-- synchronized link data to be sent to other chips through ddr_link
		link1_out:	out	std_logic_vector(27 downto 0);
		link2_out:	out	std_logic_vector(27 downto 0);
		link3_out:	out	std_logic_vector(27 downto 0);
		link4_out:	out	std_logic_vector(27 downto 0);
-- singlets out
		link1_array:out	std_logic_vector(195 downto 0);
		link2_array:out	std_logic_vector(195 downto 0);
		link3_array:out	std_logic_vector(195 downto 0);
		link4_array:out	std_logic_vector(195 downto 0);
		lock:			out	std_logic;
		diag:			out	std_logic_vector(3 downto 0);
-- backplane signals
      bus_cmd:	in std_logic;
      ctrl_bus:	inout	std_logic_vector(15 downto 0)
	);

end read_in;

architecture read_in_arch of read_in is

	-- declare components...

COMPONENT link_dskew
	PORT(
		link_data : IN std_logic_vector(27 downto 0);
		link_clk : IN std_logic;
		reset : IN std_logic;
		clock : IN std_logic;
		clear : IN std_logic;
		inject : IN std_logic;
		rd_fake : IN std_logic;
		wr_fake : IN std_logic;
		w_data : IN std_logic_vector(15 downto 0);
		addr : IN std_logic_vector(15 downto 0);          
		q : OUT std_logic_vector(195 downto 0);
		link_out : OUT std_logic_vector(27 downto 0);
		diag : OUT std_logic;
		perr : OUT std_logic;
		clk_err : OUT std_logic;
		sync_err : OUT std_logic;
		period_err : OUT std_logic;
		r_data : OUT std_logic_vector(15 downto 0)
		);
END COMPONENT;
signal R_CONFU: std_logic_vector(9 downto 0);
signal R_Fake: std_logic_vector(3 downto 0);
signal cmd_dl: std_logic_vector(6 downto 0);
signal lock_cntr: std_logic_vector(5 downto 0);
signal bus_cmd_q: std_logic;
signal bus_rwq: std_logic;
signal eof_dl: std_logic;
signal clear_dl: std_logic;
signal reset_i: std_logic;
signal resetp: std_logic;
signal inject_i: std_logic;
signal inject: std_logic;
signal bus_oe_n: std_logic;
signal rd_version: std_logic;
signal rd_eqs_vers: std_logic;
signal rd_veto_vers: std_logic;
signal vers_sect: std_logic_vector(15 downto 0);
signal ctrl_bus_q: std_logic_vector(15 downto 0);
signal addr: std_logic_vector(15 downto 0);
signal w_data: std_logic_vector(15 downto 0);
signal r_data: std_logic_vector(15 downto 0);
signal bus_o: std_logic_vector(15 downto 0);
signal bus_t: std_logic_vector(15 downto 0);
signal rd_fake: std_logic_vector(3 downto 0);
signal wr_fake: std_logic_vector(3 downto 0);
signal link_clk: std_logic_vector(3 downto 0);
signal link_sync: std_logic_vector(3 downto 0);
signal link_period: std_logic_vector(3 downto 0);
signal link_perr: std_logic_vector(3 downto 0);
signal perr: std_logic_vector(3 downto 0);
signal sync_err: std_logic_vector(3 downto 0);
signal period_err: std_logic_vector(3 downto 0);
signal clk_err: std_logic_vector(3 downto 0);
signal diag0_d0: std_logic;
signal diag0_d1: std_logic;
signal diag_sel: std_logic_vector(3 downto 0);
signal diagp: std_logic_vector(3 downto 0);
signal diag_link: std_logic_vector(3 downto 0);
signal fx: std_logic;
signal l1: std_logic;
signal clock_n: std_logic;
signal lnk_clk: std_logic;
signal test_bits: std_logic_vector(1 downto 0);
signal toggle: std_logic;
attribute IOB: string;
attribute IOB of bus_t: signal is "true";
attribute IOB of ctrl_bus_q: signal is "true";
begin
	-- combinatorial stuff...
R_CONFU <= R_CONFU7(9 downto 0) when u9 = '0' else R_CONFU9(9 downto 0);
R_Fake <= R_up_Fake(9 downto 6) when u9 = '0' else R_down_Fake(9 downto 6);
reset <= resetp;
lock <= lock_cntr(5);
g_ctrl_bus: for i in bus_o'range generate
  ctrl_bus(i) <= bus_o(i) when bus_t(i) = '0' else 'Z';
end generate;
	-- sequential stuff...
i_eof_dl: srlc16 port map(
            d => clear,
				clk => clock,
				a0 => vcc,
				a1 => gnd,
				a2 => vcc,
				a3 => gnd,
				q => eof_dl);
process(clock)
begin
  if(clock'event and clock = '1')then
    bus_cmd_q <= bus_cmd;
    if(cmd_dl(0) = '1' and bus_cmd_q = '0')then
	   lock_cntr <= (others => '0');
	 elsif(lock_cntr(5) = '0')then
	   lock_cntr <= lock_cntr + 1;
	 end if;
    if(cmd_dl = "0000000" or (cmd_dl(0) = '1' and bus_cmd_q = '0'))then
	   cmd_dl <= "0000100";
	 else
	   cmd_dl <= cmd_dl(5 downto 0) & cmd_dl(6);
	 end if;
    if(cmd_dl(0) = '1')then
-- command cycle is cycle(6)
	   diag_sel <= ctrl_bus_q(11 downto 8);
	   inject_i <= ctrl_bus_q(5);
	   reset_i <= ctrl_bus_q(4);
	 end if;
	 if(eof_dl = '1')then
		inject <= inject_i;
		resetp <= not reset_i;
	 end if;
--    if(cmd_dl(0) = '1')then
-- address cycle follows command cycle
--	   addr <= ctrl_bus;
--	   addr <= ctrl_bus(15) & "00000" & R_Fake(3 downto 1) & ctrl_bus(14 downto 8);
--	 end if;
--    if(cmd_dl(1) = '1')then
-- data cycle follows address cycle if any
--	   w_data <= ctrl_bus;
--	 end if;
	 ctrl_bus_q <= ctrl_bus;
	 bus_o <= r_data;
	 bus_rwq <= cmd_dl(0) and bus_cmd_q;
	 bus_oe_n <= not(lock_cntr(5) and ((bus_rwq and not ctrl_bus_q(15)) or cmd_dl(2)));
	 for i in bus_t'range loop
	   bus_t(i) <= bus_oe_n;
	 end loop;
	 if(bus_rwq = '1' and ctrl_bus_q(15) = '0' and ctrl_bus_q(9 downto 0) = R_CONFU)then
	   rd_version <= '1';
	 else
	   rd_version <= '0';
	 end if;
	 if(bus_rwq = '1' and ctrl_bus_q(15) = '0' and ctrl_bus_q(9 downto 6) = R_Fake)then
	   rd_fake(0) <= not ctrl_bus_q(5) and not ctrl_bus_q(4);
	   rd_fake(1) <= not ctrl_bus_q(5) and ctrl_bus_q(4);
	   rd_fake(2) <= ctrl_bus_q(5) and not ctrl_bus_q(4)and not(ctrl_bus_q(3) and ctrl_bus_q(2) and ctrl_bus_q(1));
	   rd_fake(3) <= ctrl_bus_q(5) and ctrl_bus_q(4) and not(ctrl_bus_q(3) and ctrl_bus_q(2) and ctrl_bus_q(1));
	   rd_veto_vers <= ctrl_bus_q(5) and not ctrl_bus_q(4) and ctrl_bus_q(3) and ctrl_bus_q(2) and ctrl_bus_q(1);
	   rd_eqs_vers <= ctrl_bus_q(5) and ctrl_bus_q(4) and ctrl_bus_q(3) and ctrl_bus_q(2) and ctrl_bus_q(1);
	 else
	   rd_fake <= "0000";
		rd_veto_vers <= '0';
		rd_eqs_vers <= '0';
	 end if;
	 if(bus_rwq = '1' and ctrl_bus_q(15) = '1' and ctrl_bus_q(9 downto 6) = R_Fake)then
	   wr_fake(0) <= not ctrl_bus_q(5) and not ctrl_bus_q(4);
	   wr_fake(1) <= not ctrl_bus_q(5) and ctrl_bus_q(4);
	   wr_fake(2) <= ctrl_bus_q(5) and not ctrl_bus_q(4);
	   wr_fake(3) <= ctrl_bus_q(5) and ctrl_bus_q(4);
	 else
	   wr_fake <= "0000";
	 end if;
	 if(cmd_dl(3) = '1')then
	   link_clk <= clk_err;
	   link_sync <= sync_err;
	   link_period <= period_err;
	   link_perr <= perr;
	 else
	   link_clk <= link_clk or clk_err;
	   link_sync <= link_sync or sync_err;
	   link_period <= link_period or period_err;
	   link_perr <= link_perr or perr;
	 end if;
  end if;
end process;
r_data <= link_clk & link_sync & link_period & link_perr when cmd_dl(3) = '1' else (others => 'Z');
r_data <= version when rd_version = '1' else (others => 'Z');
r_data <= vers_sect when rd_eqs_vers = '1' else (others => 'Z');
r_data <= veto_vers when rd_veto_vers = '1' else (others => 'Z');
process(clock)
begin
  if(clock'event and clock = '1')then
	 if(ctrl_bus_q(0) = '0')then
	   vers_sect <= eqs_vers(15 downto 0);
	 else
	   vers_sect <= eqs_vers(31 downto 16);
	 end if;
  end if;
end process;
	-- instantiate components...
addr <= ctrl_bus_q;
w_data <= ctrl_bus_q;
	link_mod0: link_dskew	-- this link has the error checking on it!
		port map(
			link_data => link1,
			link_clk => link1_clk,
			reset => resetp,
			clock => clock,
			clear => clear,
			inject => inject,
			q => link1_array,
			link_out => link1_out,
			diag => diag_link(0),
			perr => perr(0),
		   clk_err => clk_err(0),
		   sync_err => sync_err(0),
		   period_err => period_err(0),
			rd_fake => rd_fake(0),
			wr_fake => wr_fake(0),
			w_data => w_data,
			r_data => r_data,
			addr => addr
	);
	link_mod2: link_dskew	-- this link has the error checking on it!
		port map(
			link_data => link2,
			link_clk => link2_clk,
			reset => resetp,
			clock => clock,
			clear => clear,
			inject => inject,
			q => link2_array,
			link_out => link2_out,
			diag => diag_link(1),
			perr => perr(1),
		   clk_err => clk_err(1),
		   sync_err => sync_err(1),
		   period_err => period_err(1),
			rd_fake => rd_fake(1),
			wr_fake => wr_fake(1),
			w_data => w_data,
			r_data => r_data,
			addr => addr
	);
	link_mod3: link_dskew	-- this link has the error checking on it!
		port map(
			link_data => link3,
			link_clk => link3_clk,
			reset => resetp,
			clock => clock,
			clear => clear,
			inject => inject,
			q => link3_array,
			link_out => link3_out,
			diag => diag_link(2),
			perr => perr(2),
		   clk_err => clk_err(2),
		   sync_err => sync_err(2),
		   period_err => period_err(2),
			rd_fake => rd_fake(2),
			wr_fake => wr_fake(2),
			w_data => w_data,
			r_data => r_data,
			addr => addr
	);
	link_mod4: link_dskew	-- this link has the error checking on it!
		port map(
			link_data => link4,
			link_clk => link4_clk,
			reset => resetp,
			clock => clock,
			clear => clear,
			inject => inject,
			q => link4_array,
			link_out => link4_out,
			diag => diag_link(3),
			perr => perr(3),
		   clk_err => clk_err(3),
		   sync_err => sync_err(3),
		   period_err => period_err(3),
			rd_fake => rd_fake(3),
			wr_fake => wr_fake(3),
			w_data => w_data,
			r_data => r_data,
			addr => addr
	);
i_clear_dl: srlc16 port map(
            d => clear,
				clk => clock,
				a0 => vcc,
				a1 => vcc,
				a2 => gnd,
				a3 => gnd,
				q => clear_dl);
process(clock)
begin
  if(clock'event and clock = '1')then
	 case diag_sel is
	   when "0001" =>
		  diagp(0) <= clear_dl;
	   when "0010" =>
		  diagp(0) <= clear_dl;
	   when "0011" =>
		  diagp(0) <= clear_dl;
	   when "0100" =>
		  diagp(0) <= clear_dl;
	   when "0101" =>
		  diagp(0) <= clear_dl;
	   when others =>
		  diagp(0) <= gnd;
	 end case;
  end if;
end process;
-- output clock signal when diag_sel = "1111"
diag0_d0 <= '1' when diag_sel = "1111" else diagp(0);
diag0_d1 <= '0' when diag_sel = "1111" else diagp(0);
clock_n <= not clock;
diag0:  fddrrse
	      port map(
			    d0 => diag0_d0,
				 d1 => diag0_d1,
				 c0 => clock,
				 c1 => clock_n,
				 ce => vcc,
				 r  => gnd,
				 s  => gnd,
				 q  => diag(0));
-- this signal should not be clocked
process(diag_sel,diag_link)
begin
  case diag_sel is
	 when "0001" =>
		diag(3) <= diag_link(0);
	 when "0010" =>
		diag(3) <= diag_link(1);
	 when "0011" =>
		diag(3) <= diag_link(2);
	 when "0100" =>
		diag(3) <= diag_link(3);
	 when "0101" =>
		diag(3) <= clear_dl;
	 when others =>
		diag(3) <= gnd;
  end case;
end process;
-- select the link with control bits
lnk_clk <= link2_clk when U9 = '0' else link1_clk;
test_bits <= link2(3) & link2(1) when U9 = '0' else link1(3) & link1(1);
process(lnk_clk)
begin
  if(lnk_clk'event and lnk_clk = '1')then
    toggle <= not toggle;
	 case diag_sel is
	   when "0011" =>
		  diagp(2 downto 1) <= test_bits;
	   when "0101" =>
		  diagp(2 downto 1) <= toggle & toggle;
	   when others =>
		  diagp(2 downto 1) <= "00";
	 end case;
	 diag(2 downto 1) <= diagp(2 downto 1);
  end if;
end process;
end read_in_arch;
