-- sprior32.vhd
-- 32-bit synchronous "peel away" priority encoder with one hot outputs.
-- jamieson olsen <jamieson@fnal.gov>

-- *** revision history ***

-- 29 june 1999 	use output of priority logic to clear the input
-- 					flops.  note: this destroys the contents of the input register.

-- 14 july 1999 	for max, high, and med threshold bins

-- 21 july 1999 	yet another revision of the 16 bit version of this file.
--  				input register has a three way mux that can: load, clear, and hold.
--  				the register is cleared by the output of the select logic.  this should
--  				greatly improve speed.  note: input register is destroyed during normal
--  				readout.

-- 15 march 2000	the priority is assigned backwards!  bit 0 (block 0) 
-- 					should have a higher priority that bit 15 (block 15).  make the change to the
-- 					combinatorial "pri_out <=" section below..

-- 10 jan 2002		expand the 16-bit version up to 32-bits.

-- how it works: a normal priority encoder will report the position of the highest priority bit
-- while ignoring other "less important" bits.  This module works in much the same way, except
-- that with each clock cycle the most imporant bit is reported and then cleared, revealing 
-- the less important bits as time goes on.  For example,  if this module is loaded with
-- 0x40060012 and the LSB is given the highest priority the output will be this:

-- 0x00000002
-- 0x00000010
-- 0x00020000
-- 0x00040000
-- 0x40000000
-- 0x00000000 ...

--  01 Jul 2004:    changed to synchronous reset S.X.Wu

library ieee;
use ieee.std_logic_1164.all;

entity sprior32 is
 port (
	din: 		in std_logic_vector(31 downto 0);
	clock:		in std_logic;
	load:		in std_logic;
	reset:		in std_logic;
	dout: 		out std_logic_vector(31 downto 0)
    );
end sprior32;


architecture sprior32_arch of sprior32 is

	-- define registers...

	signal in_reg: 	std_logic_vector(31 downto 0);
	signal out_reg: std_logic_vector(31 downto 0);

	-- define signals...

	signal pri_out:	std_logic_vector(31 downto 0);

 begin

	-- combinatorial stuff

		dout(31 downto 0) <= out_reg(31 downto 0);
	

		-- 32-bit fully combinatorial one hot priority encoder
		-- bit 0 has the highest priority.
								
			pri_out( 0) <= '1' when (in_reg(0)='1') else '0';
			pri_out( 1) <= '1' when (in_reg( 1 downto 0)="10") else '0';
			pri_out( 2) <= '1' when (in_reg( 2 downto 0)="100") else '0';
			pri_out( 3) <= '1' when (in_reg( 3 downto 0)="1000") else '0';
			pri_out( 4) <= '1' when (in_reg( 4 downto 0)="10000") else '0';
			pri_out( 5) <= '1' when (in_reg( 5 downto 0)="100000") else '0';
			pri_out( 6) <= '1' when (in_reg( 6 downto 0)="1000000") else '0';
			pri_out( 7) <= '1' when (in_reg( 7 downto 0)="10000000") else '0';
			pri_out( 8) <= '1' when (in_reg( 8 downto 0)="100000000") else '0';
			pri_out( 9) <= '1' when (in_reg( 9 downto 0)="1000000000") else '0';
			pri_out(10) <= '1' when (in_reg(10 downto 0)="10000000000") else '0';
			pri_out(11) <= '1' when (in_reg(11 downto 0)="100000000000") else '0';
			pri_out(12) <= '1' when (in_reg(12 downto 0)="1000000000000") else '0';
			pri_out(13) <= '1' when (in_reg(13 downto 0)="10000000000000") else '0';
			pri_out(14) <= '1' when (in_reg(14 downto 0)="100000000000000") else '0';
			pri_out(15) <= '1' when (in_reg(15 downto 0)="1000000000000000") else '0';
			pri_out(16) <= '1' when (in_reg(16 downto 0)="10000000000000000") else '0';
			pri_out(17) <= '1' when (in_reg(17 downto 0)="100000000000000000") else '0';
			pri_out(18) <= '1' when (in_reg(18 downto 0)="1000000000000000000") else '0';
			pri_out(19) <= '1' when (in_reg(19 downto 0)="10000000000000000000") else '0';
			pri_out(20) <= '1' when (in_reg(20 downto 0)="100000000000000000000") else '0';
			pri_out(21) <= '1' when (in_reg(21 downto 0)="1000000000000000000000") else '0';
			pri_out(22) <= '1' when (in_reg(22 downto 0)="10000000000000000000000") else '0';
			pri_out(23) <= '1' when (in_reg(23 downto 0)="100000000000000000000000") else '0';
			pri_out(24) <= '1' when (in_reg(24 downto 0)="1000000000000000000000000") else '0';
			pri_out(25) <= '1' when (in_reg(25 downto 0)="10000000000000000000000000") else '0';
			pri_out(26) <= '1' when (in_reg(26 downto 0)="100000000000000000000000000") else '0';
			pri_out(27) <= '1' when (in_reg(27 downto 0)="1000000000000000000000000000") else '0';
			pri_out(28) <= '1' when (in_reg(28 downto 0)="10000000000000000000000000000") else '0';
			pri_out(29) <= '1' when (in_reg(29 downto 0)="100000000000000000000000000000") else '0';
			pri_out(30) <= '1' when (in_reg(30 downto 0)="1000000000000000000000000000000") else '0';
			pri_out(31) <= '1' when (in_reg(31 downto 0)="10000000000000000000000000000000") else '0';								
		
		
	-- sync process to maintain the output register
-- change to sychronous reset 1-jul-2004 S.X.Wu
process(clock)
begin
  if(clock'event and clock = '1')then
    if(reset = '0')then
--	   out_reg <= (others => '0');
	 else
--	   out_reg <= pri_out;
	 end if;
  end if;
end process;
out_reg <= pri_out;

--	process(clock, reset)
--	 begin
--		if (reset='0') then
--			out_reg(31 downto 0) <= X"00000000";
--		elsif rising_edge(clock) then
--			out_reg(31 downto 0) <= pri_out(31 downto 0);
--		else
--			null;
--		end if;
--	end process;




	-- sync process to maintain the input register
-- change to sychronous reset 1-jul-2004 S.X.Wu
process(clock)
begin
  if(clock'event and clock = '1')then
    if(reset = '0')then
	   in_reg <= (others => '0');
	 elsif(load = '1')then
	   in_reg <= din;
	 else
	   in_reg <= in_reg and not pri_out;
	 end if;
  end if;
end process;

--	process(clock, reset)
--	 begin
--		if (reset='0') then
--			in_reg(31 downto 0) <= X"00000000";
--
--		elsif rising_edge(clock) then
--
--			if (load='1') then	-- load
--				in_reg(31) <= din(31);
--			elsif (load='0' and pri_out(31)='1') then	-- clear
--				in_reg(31) <= '0';
--			else	-- hold
--				in_reg(31) <= in_reg(31);
--			end if;
--
--			if (load='1') then	-- load
--				in_reg(30) <= din(30);
--			elsif (load='0' and pri_out(30)='1') then	-- clear
--				in_reg(30) <= '0';
--			else	-- hold
--				in_reg(30) <= in_reg(30);
--			end if;

--			if (load='1') then	-- load
--				in_reg(29) <= din(29);
--			elsif (load='0' and pri_out(29)='1') then	-- clear
--				in_reg(29) <= '0';
--			else	-- hold
--				in_reg(29) <= in_reg(29);
--			end if;

--			if (load='1') then	-- load
--				in_reg(28) <= din(28);
--			elsif (load='0' and pri_out(28)='1') then	-- clear
--				in_reg(28) <= '0';
--			else	-- hold
--				in_reg(28) <= in_reg(28);
--			end if;

--			if (load='1') then	-- load
--				in_reg(27) <= din(27);
--			elsif (load='0' and pri_out(27)='1') then	-- clear
--				in_reg(27) <= '0';
--			else	-- hold
--				in_reg(27) <= in_reg(27);
--			end if;

--			if (load='1') then	-- load
--				in_reg(26) <= din(26);
--			elsif (load='0' and pri_out(26)='1') then	-- clear
--				in_reg(26) <= '0';
--			else	-- hold
--				in_reg(26) <= in_reg(26);
--			end if;

--			if (load='1') then	-- load
--				in_reg(25) <= din(25);
--			elsif (load='0' and pri_out(25)='1') then	-- clear
--				in_reg(25) <= '0';
--			else	-- hold
--				in_reg(25) <= in_reg(25);
--			end if;

--			if (load='1') then	-- load
--				in_reg(24) <= din(24);
--			elsif (load='0' and pri_out(24)='1') then	-- clear
--				in_reg(24) <= '0';
--			else	-- hold
--				in_reg(24) <= in_reg(24);
--			end if;

--			if (load='1') then	-- load
--				in_reg(23) <= din(23);
--			elsif (load='0' and pri_out(23)='1') then	-- clear
--				in_reg(23) <= '0';
--			else	-- hold
--				in_reg(23) <= in_reg(23);
--			end if;

--			if (load='1') then	-- load
--				in_reg(22) <= din(22);
--			elsif (load='0' and pri_out(22)='1') then	-- clear
--				in_reg(22) <= '0';
--			else	-- hold
--				in_reg(22) <= in_reg(22);
--			end if;

--			if (load='1') then	-- load
--				in_reg(21) <= din(21);
--			elsif (load='0' and pri_out(21)='1') then	-- clear
--				in_reg(21) <= '0';
--			else	-- hold
--				in_reg(21) <= in_reg(21);
--			end if;

--			if (load='1') then	-- load
--				in_reg(20) <= din(20);
--			elsif (load='0' and pri_out(20)='1') then	-- clear
--				in_reg(20) <= '0';
--			else	-- hold
--				in_reg(20) <= in_reg(20);
--			end if;

--			if (load='1') then	-- load
--				in_reg(19) <= din(19);
--			elsif (load='0' and pri_out(19)='1') then	-- clear
--				in_reg(19) <= '0';
--			else	-- hold
--				in_reg(19) <= in_reg(19);
--			end if;

--			if (load='1') then	-- load
--				in_reg(18) <= din(18);
--			elsif (load='0' and pri_out(18)='1') then	-- clear
--				in_reg(18) <= '0';
--			else	-- hold
--				in_reg(18) <= in_reg(18);
--			end if;

--			if (load='1') then	-- load
--				in_reg(17) <= din(17);
--			elsif (load='0' and pri_out(17)='1') then	-- clear
--				in_reg(17) <= '0';
--			else	-- hold
--				in_reg(17) <= in_reg(17);
--			end if;

--			if (load='1') then	-- load
--				in_reg(16) <= din(16);
--			elsif (load='0' and pri_out(16)='1') then	-- clear
--				in_reg(16) <= '0';
--			else	-- hold
--				in_reg(16) <= in_reg(16);
--			end if;

--			if (load='1') then	-- load
--				in_reg(15) <= din(15);
--			elsif (load='0' and pri_out(15)='1') then	-- clear
--				in_reg(15) <= '0';
--			else	-- hold
--				in_reg(15) <= in_reg(15);
--			end if;

--			if (load='1') then	-- load
--				in_reg(14) <= din(14);
--			elsif (load='0' and pri_out(14)='1') then	-- clear
--				in_reg(14) <= '0';
--			else	-- hold
--				in_reg(14) <= in_reg(14);
--			end if;

--			if (load='1') then	-- load
--				in_reg(13) <= din(13);
--			elsif (load='0' and pri_out(13)='1') then	-- clear
--				in_reg(13) <= '0';
--			else	-- hold
--				in_reg(13) <= in_reg(13);
--			end if;

--			if (load='1') then	-- load
--				in_reg(12) <= din(12);
--			elsif (load='0' and pri_out(12)='1') then	-- clear
--				in_reg(12) <= '0';
--			else	-- hold
--				in_reg(12) <= in_reg(12);
--			end if;

--			if (load='1') then	-- load
--				in_reg(11) <= din(11);
--			elsif (load='0' and pri_out(11)='1') then	-- clear
--				in_reg(11) <= '0';
--			else	-- hold
--				in_reg(11) <= in_reg(11);
--			end if;

--			if (load='1') then	-- load
--				in_reg(10) <= din(10);
--			elsif (load='0' and pri_out(10)='1') then	-- clear
--				in_reg(10) <= '0';
--			else	-- hold
--				in_reg(10) <= in_reg(10);
--			end if;

--			if (load='1') then	-- load
--				in_reg(9) <= din(9);
--			elsif (load='0' and pri_out(9)='1') then	-- clear
--				in_reg(9) <= '0';
--			else	-- hold
--				in_reg(9) <= in_reg(9);
--			end if;

--			if (load='1') then	-- load
--				in_reg(8) <= din(8);
--			elsif (load='0' and pri_out(8)='1') then	-- clear
--				in_reg(8) <= '0';
--			else	-- hold
--				in_reg(8) <= in_reg(8);
--			end if;

--			if (load='1') then	-- load
--				in_reg(7) <= din(7);
--			elsif (load='0' and pri_out(7)='1') then	-- clear
--				in_reg(7) <= '0';
--			else	-- hold
--				in_reg(7) <= in_reg(7);
--			end if;

--			if (load='1') then	-- load
--				in_reg(6) <= din(6);
--			elsif (load='0' and pri_out(6)='1') then	-- clear
--				in_reg(6) <= '0';
--			else	-- hold
--				in_reg(6) <= in_reg(6);
--			end if;

--			if (load='1') then	-- load
--				in_reg(5) <= din(5);
--			elsif (load='0' and pri_out(5)='1') then	-- clear
--				in_reg(5) <= '0';
--			else	-- hold
--				in_reg(5) <= in_reg(5);
--			end if;

--			if (load='1') then	-- load
--				in_reg(4) <= din(4);
--			elsif (load='0' and pri_out(4)='1') then	-- clear
--				in_reg(4) <= '0';
--			else	-- hold
--				in_reg(4) <= in_reg(4);
--			end if;

--			if (load='1') then	-- load
--				in_reg(3) <= din(3);
--			elsif (load='0' and pri_out(3)='1') then	-- clear
--				in_reg(3) <= '0';
--			else	-- hold
--				in_reg(3) <= in_reg(3);
--			end if;

--			if (load='1') then	-- load
--				in_reg(2) <= din(2);
--			elsif (load='0' and pri_out(2)='1') then	-- clear
--				in_reg(2) <= '0';
--			else	-- hold
--				in_reg(2) <= in_reg(2);
--			end if;

--			if (load='1') then	-- load
--				in_reg(1) <= din(1);
--			elsif (load='0' and pri_out(1)='1') then	-- clear
--				in_reg(1) <= '0';
--			else	-- hold
--				in_reg(1) <= in_reg(1);
--			end if;

--			if (load='1') then	-- load
--				in_reg(0) <= din(0);
--			elsif (load='0' and pri_out(0)='1') then	-- clear
--				in_reg(0) <= '0';
--			else	-- hold
--				in_reg(0) <= in_reg(0);
--			end if;

--		end if;
--	end process;


end sprior32_arch;
