-- extrapolate.vhd
-- jamieson olsen <jamieson@fnal.gov>

-- 	This module extrapolates MAX Pt Bin CFT tracks to the CPS axial layer.  
--	This extrapolation is done in a two stage pipeline.  The first stage
-- 	considers only the track's PHI value and projects an infinite Pt track
-- 	onto the CPS layer.  The next step involves shifted the infinite projected 
--	track by an offset value.  This offset value is a function of the Pt theshold
--	bin and the track curvature.

-- 	tracks come in with format SVCEeeePppppp; S=start, V=Valid, C=curve, 
--	Eeee=extended Pt, and Pppppp=track phi.

--	reference eng. note 2002-02-06A

-- *** revision history *** 
--
-- 06 Feb 2002:	  	created and functionally simulated.  does NOT include outputs 
--					to L2 RAMs.
-- 08 Jul 2002:		change Pt bin definition so that 00=max, 11-low.
-- 12 Aug 2002:		modify bus width, add separate start input
-- 10 Mar 2003:		change track phi from 1-44 to 0-43  (bug found by Makoto Tomoto)
-- 13 May 2003:		consistant curve def 1=CCW

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity extrapolate is
	port(
		reset:		in 	std_logic;							-- active low async
		clock:		in 	std_logic;							-- dfe board clock 53mhz
		start_in:	in	std_logic;							-- set for mark first track in
		track:		in	std_logic_vector(10 downto 0);		-- format VCEeePppppp
		pt_bin:		in	std_logic_vector( 1 downto 0);		-- 00=max, 01=high, 10=med, 11=low
		proj:		out	std_logic_vector( 4 downto 0);		-- projected track phi
		valid:		out	std_logic;							-- set if track is valid
		curve:		out	std_logic;							-- set if track curves CW (decreasing phi)
		start_out:	out	std_logic							-- set to mark the first track leaving this module
	);
end extrapolate;

architecture extrapolate_arch of extrapolate is

	-- define signals...

	signal inf_proj:		std_logic_vector(4 downto 0);
	signal offset:			std_logic_vector(4 downto 0);
	signal shift_cw:		std_logic_vector(4 downto 0);	-- sub offset (curve = 0 = CW)
	signal shift_ccw:		std_logic_vector(4 downto 0);	-- add offset (curve = 1 = CCW)
	
	-- define registers...

	signal proj_reg:		std_logic_vector(4 downto 0);
	signal inf_reg:			std_logic_vector(4 downto 0);
	signal curve_reg:		std_logic_vector(1 downto 0);
	signal valid_reg:		std_logic_vector(1 downto 0);
	signal start_reg:		std_logic_vector(1 downto 0);
	
 begin

	-- combinatorial stuff...

		proj(4 downto 0)	<= proj_reg(4 downto 0);
		valid				<= valid_reg(1);
		curve				<= curve_reg(1);
		start_out			<= start_reg(1);

		-- this lookup table derived from excel spreadsheet 2002-01-31A.xls
		-- changed to phi = 0-43  10-Mar-2003 

		inf_proj(4 downto 0) <=	"01000" when (track(5 downto 0)="000000") else
								"01000" when (track(5 downto 0)="000001") else
								"01001" when (track(5 downto 0)="000010") else
								"01001" when (track(5 downto 0)="000011") else
								"01001" when (track(5 downto 0)="000100") else
								"01010" when (track(5 downto 0)="000101") else
								"01010" when (track(5 downto 0)="000110") else
								"01010" when (track(5 downto 0)="000111") else
								"01011" when (track(5 downto 0)="001000") else
								"01011" when (track(5 downto 0)="001001") else
								"01011" when (track(5 downto 0)="001010") else
								"01100" when (track(5 downto 0)="001011") else
								"01100" when (track(5 downto 0)="001100") else
								"01100" when (track(5 downto 0)="001101") else
								"01101" when (track(5 downto 0)="001110") else
								"01101" when (track(5 downto 0)="001111") else
								"01110" when (track(5 downto 0)="010000") else
								"01110" when (track(5 downto 0)="010001") else
								"01110" when (track(5 downto 0)="010010") else
								"01111" when (track(5 downto 0)="010011") else
								"01111" when (track(5 downto 0)="010100") else
								"01111" when (track(5 downto 0)="010101") else
								"10000" when (track(5 downto 0)="010110") else
								"10000" when (track(5 downto 0)="010111") else
								"10000" when (track(5 downto 0)="011000") else
								"10001" when (track(5 downto 0)="011001") else
								"10001" when (track(5 downto 0)="011010") else
								"10001" when (track(5 downto 0)="011011") else
								"10010" when (track(5 downto 0)="011100") else
								"10010" when (track(5 downto 0)="011101") else
								"10011" when (track(5 downto 0)="011110") else
								"10011" when (track(5 downto 0)="011111") else
								"10011" when (track(5 downto 0)="100000") else
								"10100" when (track(5 downto 0)="100001") else
								"10100" when (track(5 downto 0)="100010") else
								"10100" when (track(5 downto 0)="100011") else
								"10101" when (track(5 downto 0)="100100") else
								"10101" when (track(5 downto 0)="100101") else
								"10101" when (track(5 downto 0)="100110") else
								"10110" when (track(5 downto 0)="100111") else
								"10110" when (track(5 downto 0)="101000") else
								"10110" when (track(5 downto 0)="101001") else
								"10111" when (track(5 downto 0)="101010") else
								"10111" when (track(5 downto 0)="101011") else
								"00000";

--		offset(4 downto 0)		<=	"00000" when (pt_bin(1 downto 0)="00") else		-- Pt = max
--									"00001" when (pt_bin(1 downto 0)="01") else		-- Pt = high
--									"00010" when (pt_bin(1 downto 0)="10") else		-- Pt = med
--									"00011";										-- Pt = low
-- changed 11-30-04
		offset(4 downto 0)		<=	"00001" when (pt_bin(1 downto 0)="00") else		-- Pt = max
									"00010" when (pt_bin(1 downto 0)="01") else		-- Pt = high
									"00011" when (pt_bin(1 downto 0)="10") else		-- Pt = med
									"00101";										-- Pt = low

		shift_cw(4 downto 0)	<= inf_reg(4 downto 0) - offset(4 downto 0);

		shift_ccw(4 downto 0)	<= inf_reg(4 downto 0) + offset(4 downto 0);

	-- sequential stuff...

--	maintain_reg: process(clock, reset)	  	change to sync reset Aug-06-04
	maintain_reg: process(clock)
 	 begin

		if rising_edge(clock) then
		 if (reset='0') then 
			inf_reg(4 downto 0)		<= "00000";
			curve_reg(1 downto 0)	<= "00";
			valid_reg(1 downto 0)	<= "00";
			start_reg(1 downto 0)	<= "00";
			proj_reg(4 downto 0)	<= "00000";
		 else
			inf_reg(4 downto 0)	<= inf_proj(4 downto 0);

			curve_reg(1)		<= curve_reg(0);
			curve_reg(0)		<= track(9);		

			valid_reg(1)		<= valid_reg(0);
			valid_reg(0)		<= track(10);

			start_reg(1)		<= start_reg(0);
			start_reg(0)		<= start_in;

			if (curve_reg(0)='1') then
				proj_reg(4 downto 0)	<= shift_ccw(4 downto 0);
			else
				proj_reg(4 downto 0)	<= shift_cw(4 downto 0);
			end if;

		 end if;
	  end if;
	end process;

end extrapolate_arch;
