-- isolation.vhd
-- jamieson olsen <jamieson@fnal.gov>

--	logic to determine if there is an isolated track in the home sector.  By definition
--	an isolated track *cannot* have any other tracks in neighhouring sectors.  To 
--	support this, the DFE motherboard supports communication between adjecent DFEA 
--	daughterboards.

--	The first step is for this DFEA to determine how many tracks are present in the
--	current event.  If the number of tracks is equal to or greater than 1 the home 
--	DFEA sets a bit that's available to its PREV and NEXT neighbouring sectors.  Likewise,
--	the HOME DFEA "sees" the "One or More Tracks" bit from its PREV and NEXT neighbours.

--	The HOME DFEA reports an isolated track the following conditions are met:
--	1) The home sector has only ONE track
--	2) The PREV and NEXT sectors have NO tracks.

--	* timing issues *

--	There may be some skew between DFEAs, also there may be delays caused by cables and 
--	buffer chips on the motherboard.  The longest path between two adjecent DFEAs will 
--	be 2.7ns (tx) + 10ns (1m cable delay) + 4.7ns (rx) = about one 53MHz clock cycle.  
-- 	To be really rock solid this module will assert the "one or more tracks" bit for an
--	entire event -- 132ns.  Three clocks after asserting this bit it will sample the
--	"one or more tracks" bits from the neightbours.  Should completely take out any 
--	skew between boards and any cable slop!

--	reference my engineering note 2002-02-07C!

-- *** revision history *** 
--
-- 06 Feb 2002:	  	created and functionally simulated.
--					
-- 21 Feb 2002:		adjust delay to match up with other modules in L1 pipeline.


library ieee;
use ieee.std_logic_1164.all;

entity isolation is
	port(
		reset:		in 	std_logic;		--	active low
		clock:		in 	std_logic;		
		max_val:	in	std_logic;		-- 	track valid bit, active high
		high_val:	in	std_logic;
		med_val:	in	std_logic;
		low_val:	in	std_logic;
		start:		in	std_logic;		--	set to mark the first tracks of an event
		prev_trk:	in	std_logic;		-- 	set if one or more tracks are in the PREV sector
		next_trk:	in	std_logic;		-- 	set if one or more tracks are in the NEXT sector
		trk:		out	std_logic;		-- 	set if one or more tracks present in HOME sector
										--	send to PREV and NEXT
		iso_trk:	out	std_logic		--	set if the track is isolated
	);
end isolation;

architecture isolation_arch of isolation is

	-- define signals...

		signal temp_trk:		std_logic;
		signal one_hot:			std_logic;
		signal nor_val:			std_logic;
		signal temp_iso:		std_logic;
		signal iso_track:		std_logic;
	
	-- define registers...

		signal trk_reg:		std_logic;
		signal start_reg:	std_logic_vector(2 downto 0);
		signal iso_reg:		std_logic_vector(5 downto 0);
		signal val_reg:		std_logic_vector(3 downto 0);
	
 begin

	-- combinatorial stuff...

		trk			<= trk_reg;

		iso_trk		<=	iso_reg(5);

		temp_trk 	<=	'1' when (max_val='1' or high_val='1' or med_val='1' or low_val='1') else 
						'0';

		one_hot		<=	'1' when (val_reg(3 downto 0)="0001") else
						'1' when (val_reg(3 downto 0)="0010") else
						'1' when (val_reg(3 downto 0)="0100") else
						'1' when (val_reg(3 downto 0)="1000") else
						'0';

		nor_val		<=	'1' when (max_val='0' and high_val='0' and med_val='0' and low_val='0') else 
						'0';

		temp_iso	<=	one_hot and nor_val;

		iso_track	<=	'1' when (prev_trk='0' and next_trk='0' and iso_reg(1)='1') else 
						'0';

	-- sequential stuff...

--	rtl: process(clock, reset)		 	change to sync reset Aug-06-04
	rtl: process(clock)
 	 begin

		if rising_edge(clock) then
		 if (reset='0') then 
			iso_reg(5 downto 0)		<= "000000";
			start_reg(2 downto 0)	<= "000";
			val_reg(3 downto 0)		<= "0000";
			trk_reg					<= '0';
		 else
			start_reg(0)	<= 	start;
			start_reg(1)	<=	start_reg(0);
			start_reg(2)	<=	start_reg(1);

			if (start='1') then
				trk_reg			<= 	temp_trk;
			else
				trk_reg			<=	trk_reg;
			end if;

			val_reg(3 downto 0)	<=	max_val & high_val & med_val & low_val;

			iso_reg(1)			<=	iso_reg(0);
			iso_reg(3)			<= 	iso_reg(2);
			iso_reg(4)			<= 	iso_reg(3);
			iso_reg(5)			<= 	iso_reg(4);

			if (start_reg(0)='1') then	
				iso_reg(0)		<=	temp_iso;
			else
				iso_reg(0)		<=	iso_reg(0);
			end if;

			if (start_reg(2)='1') then	
				iso_reg(2)		<=	iso_track;
			else
				iso_reg(2)		<=	iso_reg(2);
			end if;

		 end if;
	  end if;
	end process;

end isolation_arch;
