-- level_one_format.vhd
-- jamieson olsen <jamieson@fnal.gov>

-- Accepts data fields in parallel and formats 
-- it into a LEVEL-ONE data frame for CTOC.

-- *** revision history ***

-- 05 Feb 2002:     Matches with the "final" Level-One DFEA->CTOC
--                  format.  Parity generation is NOT supported in
--                  this module!  If an isolated track is matched 
--                  to a cluster it is assumed to be a TIGHT match.

-- 15 May 2002: 	zero out the other junk in the isolated fields 
--					if the iso_v bit is not set.

-- 08 Jul 2002:		add FIRST_XING and CFT_RESET to output.

-- 28 Aug 2002:		fix clockwise and counter-clockwise track fields (they were reversed)
--					add three fake "events".

-- 23 Jan 2003: 	change the "busy" fake event to "test" event pattern.
-- 17 Feb 2003:		fix typo in "test event"
-- 05 Mar 2003:		cosmetic changes to comments...
-- 13 May 2003:		curve defition 1=CCW

--  NOTE: CLOCK-WISE and COUNTER-CLOCK-WISE are defined by looking NORTH at the SOUTH face of
--  the CFT detector. What the protocols call "XPT+TC" is "a MAXIMUM Pt track curving CCW with
--  a matching cluster."

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity level_one_format is
    port(
        clock:      in  std_logic;                      -- 53 MHz
        reset:      in  std_logic;                      -- active low
        start:      in  std_logic;                      -- active high, set once every 7 cycles.
        pm:         in  std_logic_vector(1 downto 0);   -- pass-n-mark field
		fx:			in	std_logic;						-- FIRST_XING, active high
		cr:			in	std_logic;						-- CFT_RESET, active high
        oct:        in  std_logic_vector(2 downto 0);   -- octant field (0-7)
        rel_sect:   in  std_logic_vector(3 downto 0);   -- relative sector within a super-sector (0-15)
        max_cw:     in  std_logic_vector(2 downto 0);   -- count of CW  curving max tracks w/  cluster match
        max_ccw:    in  std_logic_vector(2 downto 0);   -- count of CCW curving max tracks w/  cluster match
        max_ncw:    in  std_logic_vector(2 downto 0);   -- count of CW  curving max tracks w/o cluster match
        max_nccw:   in  std_logic_vector(2 downto 0);   -- count of CCW curving max tracks w/o cluster match
        high_cw:    in  std_logic_vector(2 downto 0);   -- count of CW  curving high tracks w/  cluster match
        high_ccw:   in  std_logic_vector(2 downto 0);   -- count of CCW curving high tracks w/  cluster match
        high_ncw:   in  std_logic_vector(2 downto 0);   -- count of CW  curving high tracks w/o cluster match
        high_nccw:  in  std_logic_vector(2 downto 0);   -- count of CCW curving high tracks w/o cluster match
        med_cw:     in  std_logic_vector(2 downto 0);   -- count of CW  curving med tracks w/  cluster match
        med_ccw:    in  std_logic_vector(2 downto 0);   -- count of CCW curving med tracks w/  cluster match
        med_ncw:    in  std_logic_vector(2 downto 0);   -- count of CW  curving med tracks w/o cluster match
        med_nccw:   in  std_logic_vector(2 downto 0);   -- count of CCW curving med tracks w/o cluster match
        low_cw:     in  std_logic_vector(2 downto 0);   -- count of CW  curving low tracks w/  cluster match
        low_ccw:    in  std_logic_vector(2 downto 0);   -- count of CCW curving low tracks w/  cluster match
        low_ncw:    in  std_logic_vector(2 downto 0);   -- count of CW  curving low tracks w/o cluster match
        low_nccw:   in  std_logic_vector(2 downto 0);   -- count of CCW curving low tracks w/o cluster match
        clusters:   in  std_logic_vector(1 downto 0);   -- number of clusters found in the home sector
        doublets:   in  std_logic_vector(7 downto 0);   -- number of doublets found in the home sector
        sum_pt:     in  std_logic_vector(5 downto 0);   -- sum the Pt of all found tracks 
        iso_v:      in  std_logic;                      -- set if a valid isolated track exists
        iso_assoc:  in  std_logic;                      -- set if the isolated track matches to a cluster
        iso_curv:   in  std_logic;                      -- set if iso track curvature is CCW
        iso_pt:     in  std_logic_vector(1 downto 0);   -- Pt bin of the iso track ('00'=max, .. '11'=low)
        iso_oc:     in  std_logic;                      -- isolated occupancy bit?  leave zero.
		fake_sel:	in	std_logic_vector(1 downto 0);	-- select different fake events or real data
        ctoc_out:   out std_logic_vector(27 downto 0)   -- level_one formatted output.  
    );  
end level_one_format;

architecture level_one_arch of level_one_format is

    -- declare registers...

        signal word_1_reg:      std_logic_vector(27 downto 0);
        signal word_2_reg:      std_logic_vector(27 downto 0);
        signal word_3_reg:      std_logic_vector(27 downto 0);
        signal word_4_reg:      std_logic_vector(27 downto 0);
        signal word_5_reg:      std_logic_vector(27 downto 0);
        signal word_6_reg:      std_logic_vector(27 downto 0);

    -- declare signals...

        signal real_1:          std_logic_vector(27 downto 0);
        signal real_2:          std_logic_vector(27 downto 0);
        signal real_3:          std_logic_vector(27 downto 0);
        signal real_4:          std_logic_vector(27 downto 0);
        signal real_5:          std_logic_vector(27 downto 0);
        signal real_6:          std_logic_vector(27 downto 0);

        signal test_1:          std_logic_vector(27 downto 0);
        signal test_2:          std_logic_vector(27 downto 0);
        signal test_3:          std_logic_vector(27 downto 0);
        signal test_4:          std_logic_vector(27 downto 0);
        signal test_5:          std_logic_vector(27 downto 0);
        signal test_6:          std_logic_vector(27 downto 0);

        signal sparse_1:          std_logic_vector(27 downto 0);
        signal sparse_2:          std_logic_vector(27 downto 0);
        signal sparse_3:          std_logic_vector(27 downto 0);
        signal sparse_4:          std_logic_vector(27 downto 0);
        signal sparse_5:          std_logic_vector(27 downto 0);
        signal sparse_6:          std_logic_vector(27 downto 0);

        signal null_1:          std_logic_vector(27 downto 0);
        signal null_2:          std_logic_vector(27 downto 0);
        signal null_3:          std_logic_vector(27 downto 0);
        signal null_4:          std_logic_vector(27 downto 0);
        signal null_5:          std_logic_vector(27 downto 0);
        signal null_6:          std_logic_vector(27 downto 0);
    

		signal iso_byte:		std_logic_vector(7 downto 0);
        
    -- declare components...



begin

    -- combinatorial stuff...

        real_1(27 downto 0) <= "1110" & pm(1 downto 0) & oct(2 downto 0) & cr & fx & "00000" & rel_sect(3 downto 0) & "00101" & "111";
        real_2(27 downto 0) <= "0000" & "00000000" & max_ccw(2 downto 0) & max_cw(2 downto 0) & "00" & max_nccw(2 downto 0) & max_ncw(2 downto 0) & "00";
        real_3(27 downto 0) <= "0000" & "00000000" & high_ccw(2 downto 0) & high_cw(2 downto 0) & "00" & high_nccw(2 downto 0) & high_ncw(2 downto 0) & "00";
        real_4(27 downto 0) <= "0000" & "00000000" & med_ccw(2 downto 0) & med_cw(2 downto 0) & "00" & med_nccw(2 downto 0) & med_ncw(2 downto 0) & clusters(1 downto 0);
        real_5(27 downto 0) <= "0000" & "00000000" & low_ccw(2 downto 0) & low_cw(2 downto 0) & "00" & low_nccw(2 downto 0) & low_ncw(2 downto 0) & "00";
        real_6(27 downto 0) <= "0000" & sum_pt(5 downto 0) & "00" & doublets(7 downto 0) & iso_byte(7 downto 0);


		-- these are the canned fake events.  in all cases the control bits are passed through, but the octant and 
		-- sector fields are zeroed out.

			null_1(27 downto 0) <= "1110" & pm(1 downto 0) & "000" & cr & fx & "00000" & "0000" & "00101" & "111";
			null_2(27 downto 0) <= X"0000000";
			null_3(27 downto 0) <= X"0000000";
			null_4(27 downto 0) <= X"0000000";
			null_5(27 downto 0) <= X"0000000";
			null_6(27 downto 0) <= X"0000000";

			test_1(27 downto 0) <= X"E00002F";
			test_2(27 downto 0) <= X"0111111";
			test_3(27 downto 0) <= X"0222222";
			test_4(27 downto 0) <= X"0444444";
			test_5(27 downto 0) <= X"0888888";
			test_6(27 downto 0) <= X"0FFFFFF";

			sparse_1(27 downto 0) <= "1110" & pm(1 downto 0) & "000" & cr & fx & "00000" & "0000" & "00101" & "111";
			sparse_2(27 downto 0) <= X"0000000";
			sparse_3(27 downto 0) <= X"1000020";
			sparse_4(27 downto 0) <= X"1000004";
			sparse_5(27 downto 0) <= X"1000004";
			sparse_6(27 downto 0) <= X"1081800";

        ctoc_out(27 downto 0) <= word_1_reg(27 downto 0);

		-- if there is no isolated track iso_v will be cleared.  in this case zero out the junk in the other iso fields 
		-- added 15may2002 JTO.

		iso_byte(7 downto 0) <= ("100" & iso_assoc & iso_curv & iso_pt(1 downto 0) & iso_oc) when (iso_v='1') else
								 "00000000";


    -- sequential stuff...

--        load_n_shift_frame: process(clock, reset)		change to sync reset Aug-06-04
        load_n_shift_frame: process(clock)
         begin

            if rising_edge(clock) then
              if (reset='0') then

                    word_1_reg(27 downto 0) <= X"0000000";
                    word_2_reg(27 downto 0) <= X"0000000";
                    word_3_reg(27 downto 0) <= X"0000000";
                    word_4_reg(27 downto 0) <= X"0000000";
                    word_5_reg(27 downto 0) <= X"0000000";
                    word_6_reg(27 downto 0) <= X"0000000";
				  else
                if (start='1') then                                     -- load

					if (fake_sel(1 downto 0)="11") then -- load "test" fake event.
	                    word_1_reg(27 downto 0) <= test_1(27 downto 0);
	                    word_2_reg(27 downto 0) <= test_2(27 downto 0);
	                    word_3_reg(27 downto 0) <= test_3(27 downto 0);
	                    word_4_reg(27 downto 0) <= test_4(27 downto 0);
	                    word_5_reg(27 downto 0) <= test_5(27 downto 0);
	                    word_6_reg(27 downto 0) <= test_6(27 downto 0);

					elsif (fake_sel(1 downto 0)="10") then	-- load "sparse" fake event
	                    word_1_reg(27 downto 0) <= sparse_1(27 downto 0);
	                    word_2_reg(27 downto 0) <= sparse_2(27 downto 0);
	                    word_3_reg(27 downto 0) <= sparse_3(27 downto 0);
	                    word_4_reg(27 downto 0) <= sparse_4(27 downto 0);
	                    word_5_reg(27 downto 0) <= sparse_5(27 downto 0);
	                    word_6_reg(27 downto 0) <= sparse_6(27 downto 0);
					
					elsif (fake_sel(1 downto 0)="01") then	-- load "null" fake event
	                    word_1_reg(27 downto 0) <= null_1(27 downto 0);
	                    word_2_reg(27 downto 0) <= null_2(27 downto 0);
	                    word_3_reg(27 downto 0) <= null_3(27 downto 0);
	                    word_4_reg(27 downto 0) <= null_4(27 downto 0);
	                    word_5_reg(27 downto 0) <= null_5(27 downto 0);
	                    word_6_reg(27 downto 0) <= null_6(27 downto 0);

					else									-- load real data
	                    word_1_reg(27 downto 0) <= real_1(27 downto 0);
	                    word_2_reg(27 downto 0) <= real_2(27 downto 0);
	                    word_3_reg(27 downto 0) <= real_3(27 downto 0);
	                    word_4_reg(27 downto 0) <= real_4(27 downto 0);
	                    word_5_reg(27 downto 0) <= real_5(27 downto 0);
	                    word_6_reg(27 downto 0) <= real_6(27 downto 0);
					end if;

                else                                                    -- shift

                    word_1_reg(27 downto 0) <= word_2_reg(27 downto 0);
                    word_2_reg(27 downto 0) <= word_3_reg(27 downto 0);
                    word_3_reg(27 downto 0) <= word_4_reg(27 downto 0);
                    word_4_reg(27 downto 0) <= word_5_reg(27 downto 0);
                    word_5_reg(27 downto 0) <= word_6_reg(27 downto 0);
                    word_6_reg(27 downto 0) <= X"0000000";

                end if;
            end if;
			 end if;
        end process;

    -- instantiate components...


end level_one_arch;
