-- window_comp.vhd
-- jamieson olsen <jamieson@fnal.gov>
-- 31 Jan 2002

-- last stage of the track/cluster matching pipeline.  

-- two 6-bit values are clocked into this module.  Each value has the format
-- SsVDdd where: 
--	Ss  = the two LSb's of the cluster index.
-- 	V   = valid bit, active high
-- 	Ddd = the absolute difference between that cluster's centroid and 
--        the extrapolated CFT track.

-- 	this module considers a cluster and track matched if the absolute difference
-- 	between the extrapolated track and cluster centroid is WINDOW or less.

-- A(5..0)  	B(5..0)		cluster_index						valid_match
-- -------		-------		------------------------------		-----------
--  AA1yyyy		BB1zzzz		if yyyy<=zzzz pass 0AA, else 1BB	set if [smaller of yyy or zzz] <= WINDOW
--  AA1yyyy		BB0----		pass 0AA							set if yyy <= WINDOW
--  AA0----		BB1zzzz		pass 1BB							set if zzz <= WINDOW
--  --0----		--0----	    pass 000							clear

-- * if aa and bb are equal, always choose A.

--	revised 11 Feb 2002: make this generic to all Pt bins, just have to specify the 
--	matching window (0-7).

-- 	revised 14 Aug 2002: expand the input busses by 1 bit in the diffference field.  expand the window
--	by one bit, now supports windows of (0-15) CPS strips.

-- 	revised 21 Oct 2002:	add reset


library ieee;
use ieee.std_logic_1164.all;

entity window_comp is
	port(
		clock:			in	std_logic;
		reset:			in	std_logic;
		a: 				in 	std_logic_vector(6 downto 0);
		b:				in	std_logic_vector(6 downto 0);
		window:			in	std_logic_vector(3 downto 0);
		cluster_index:	out	std_logic_vector(2 downto 0);
		valid_match:	out	std_logic
	);
end window_comp;

architecture window_comp_arch of window_comp is

	-- define registers...

		signal valid_reg: 	std_logic;
		signal index_reg:	std_logic_vector(2 downto 0);

	-- define signals

		signal valid:		std_logic;
		signal smaller_a:	std_logic;
		signal sel_a:		std_logic;
		signal sel_b:		std_logic;
		signal temp_index:	std_logic_vector(2 downto 0);
	
 begin

	-- combinatorial stuff...

		-- compare values...

		smaller_a				<=	'1' when ( a(3 downto 0) < b(3 downto 0) ) else
									'1' when ( a(3 downto 0) = b(3 downto 0) ) else
									'0';

		sel_a					<= 	'1' when (a(4)='1' and b(4)='1' and smaller_a='1') else
									'1' when (a(4)='1' and b(4)='0'                  ) else
									'0';

		sel_b					<= 	'1' when (a(4)='1' and b(4)='1' and smaller_a='0') else
									'1' when (a(4)='0' and b(4)='1'                  ) else
									'0';

		temp_index(2 downto 0)	<= 	('1'&b(6 downto 5)) when (sel_a='0' and sel_b='1') else
									('0'&a(6 downto 5)) when (sel_a='1' and sel_b='0') else
									"000";

		-- HERE is where the matching window is specified!  

		valid		<= 	'1' when (sel_a='0' and sel_b='1' and (b(3 downto 0) < window(3 downto 0)) ) else
						'1' when (sel_a='0' and sel_b='1' and (b(3 downto 0) = window(3 downto 0)) ) else
						'1' when (sel_a='1' and sel_b='0' and (a(3 downto 0) < window(3 downto 0)) ) else
						'1' when (sel_a='1' and sel_b='0' and (a(3 downto 0) = window(3 downto 0)) ) else

						'0';
			
		cluster_index(2 downto 0)	<= index_reg(2 downto 0);

		valid_match					<= valid_reg;


	-- sequential stuff...

--		process(clock, reset) 	 	change to sync reset Aug-06-04
		process(clock) 
		 begin
			if rising_edge(clock) then
			 if (reset='0') then
				index_reg(2 downto 0)	<= "000";
				valid_reg				<= '0';
			 else
				index_reg(2 downto 0)	<= temp_index(2 downto 0);
				valid_reg				<= valid;
			 end if;
		  end if;
		end process;

end window_comp_arch;




