-- L2_trackram.vhd
-- part of the L2 engine
-- jamieson olsen <jamieson@fnal.gov>
-- 27 Sept 2002

-- as L2 CFT tracks are progressing down the L2engine this module looks at each
-- track.  it stores the minimal track info (Tvalid, Pt, ExtPt, curve, and CM) 
-- needed to create a L2CPS field into a small ram.

-- To store a track the following conditions must be met:
-- 1. the track is valid
-- 2. the track is matched to a cluster (CM=1)
-- 3. the memory location pointed to by CNUM is empty

-- when the track is written into the memory location pointed to by CNUM, then it is marked as
-- full and cannot be over-written by another track pointing to the same cluster.  this insures
-- that the cluster now points to the highest pt matching track.

-- assert FLUSH for at least 8 clock cycles.  this sets the flags to empty and erases the first 
-- 8 memory locations in the ram.  This should be done after L2CPS readout.

-- 13 May 2003:  consistant curve 1 = CCW
-- 3/10/2005 flush at the same time as being read out S. X. Wu
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity L2_trackram is
	port(
		clock:		in	std_logic;
		reset:		in	std_logic;

		tvalid:		in	std_logic;						-- valid track
		curve:		in	std_logic;						-- track curvature	1=CCW
		cm:			in	std_logic;						-- track is matched to a cluster
		pt:			in	std_logic_vector(1 downto 0);	-- pt bin
		extpt:		in	std_logic_vector(2 downto 0);	-- extended pt info

		cnum:		in	std_logic_vector(2 downto 0);	-- cluster index
		flush:		in	std_logic;						-- empty out the ram

		tvalid_out:	out	std_logic;
		cm_out:		out	std_logic;
		curve_out:	out	std_logic;
		pt_out:		out	std_logic_vector(1 downto 0);
		extpt_out:	out	std_logic_vector(2 downto 0)
    );
end L2_trackram;


architecture L2_trackram_arch of L2_trackram is

	-- define registers...
	
		signal empty_reg:	std_logic_vector(7 downto 0);		-- set means that the mem location(x) is empty
		signal flush_addr:	std_logic_vector(2 downto 0);
	
	-- define signals

		signal addr:		std_logic_vector(3 downto 0);
		signal din:			std_logic_vector(7 downto 0);
		signal dout:		std_logic_vector(7 downto 0);
		signal write:		std_logic_vector(7 downto 0);
		signal we:			std_logic;

	-- declare components...

		component ram8x16d
		 	port (
				wclk:		in	std_logic;						-- sync write clk
				we:			in	std_logic;						-- active high
				addra:		in	std_logic_vector(3 downto 0);	-- write port address
				din:		in	std_logic_vector(7 downto 0);
				addrb:		in	std_logic_vector(3 downto 0);	-- read port address
				dout:		out	std_logic_vector(7 downto 0)	-- read data
			);
		end component;

		
 begin

	-- combinatorial stuff...

			-- din is what gets written into the ram

			din(7 downto 0) <= (tvalid & cm & curve & pt(1 downto 0) & extpt(2 downto 0)) when flush='0' else
								"00000000";

			-- this is the write address pointer 

--			addr(3 downto 0) <= ('0' & cnum(2 downto 0)) when flush='0' else
--								('0' & flush_addr(2 downto 0));
			addr(3 downto 0) <= '0' & cnum(2 downto 0);
	
			tvalid_out            <= dout(7);
			cm_out 	              <= dout(6);
			curve_out             <= dout(5);
			pt_out(1 downto 0)    <= dout(4 downto 3);
			extpt_out(2 downto 0) <= dout(2 downto 0);

		-- determine which memory location should get written on the next clock

			write(0) <= '1' when (tvalid='1' and cm='1' and cnum(2 downto 0)="000" and empty_reg(0)='1') else 
						'0';
	
			write(1) <= '1' when (tvalid='1' and cm='1' and cnum(2 downto 0)="001" and empty_reg(1)='1') else 
						'0';
	
			write(2) <= '1' when (tvalid='1' and cm='1' and cnum(2 downto 0)="010" and empty_reg(2)='1') else 
						'0';
	
			write(3) <= '1' when (tvalid='1' and cm='1' and cnum(2 downto 0)="011" and empty_reg(3)='1') else 
						'0';
	
			write(4) <= '1' when (tvalid='1' and cm='1' and cnum(2 downto 0)="100" and empty_reg(4)='1') else 
						'0';
	
			write(5) <= '1' when (tvalid='1' and cm='1' and cnum(2 downto 0)="101" and empty_reg(5)='1') else 
						'0';
	
			write(6) <= '1' when (tvalid='1' and cm='1' and cnum(2 downto 0)="110" and empty_reg(6)='1') else 
						'0';
	
			write(7) <= '1' when (tvalid='1' and cm='1' and cnum(2 downto 0)="111" and empty_reg(7)='1') else 
						'0';

		-- tell the ram to write the current track info into the RAM

			-- when flushing, enable writes into the ram.

			we <= 	flush or (write(0) or write(1) or write(2) or write(3) or
					write(4) or write(5) or write(6) or write(7));

	-- sequential stuff...

--		empty_control: process(clock, reset)	 change to sync reset Aug-06-04
		empty_control: process(clock)
 		 begin
			if rising_edge(clock) then
			 if (reset='0') then
				empty_reg(7 downto 0) <= "11111111";
--				flush_addr(2 downto 0) <= "000";
			 else
				if (flush='1') then
					empty_reg(7 downto 0) <= "11111111";	-- all mem locations are empty
--					flush_addr(2 downto 0) <= flush_addr(2 downto 0) + 1;
				else
					empty_reg(0) <= empty_reg(0) and (not write(0));		-- if a mem location is written, clear its empty bit
					empty_reg(1) <= empty_reg(1) and (not write(1));
					empty_reg(2) <= empty_reg(2) and (not write(2));
					empty_reg(3) <= empty_reg(3) and (not write(3));
					empty_reg(4) <= empty_reg(4) and (not write(4));
					empty_reg(5) <= empty_reg(5) and (not write(5));
					empty_reg(6) <= empty_reg(6) and (not write(6));
					empty_reg(7) <= empty_reg(7) and (not write(7));						
				end if;
			 end if;
		  end if;
		end process;
	
	-- instantiate components...

		track_ram: ram8x16d
		 	port map(
				wclk		      => clock,
				we				  => we,
				addra(3 downto 0) => addr(3 downto 0),
				din(7 downto 0)	  => din(7 downto 0),
				addrb(3 downto 0) => addr(3 downto 0),
				dout(7 downto 0)  => dout(7 downto 0)
			);

end L2_trackram_arch;

