-- L2CPS_format.vhd
-- jamieson olsen <jamieson@fnal.gov>

-- 	format the L2 CPS record.  build up the header frames
-- 	and data frames.  cluster_count is the number of clusters reported by the L2_CLUSTSORT 
--  module.  Clusters supplied to this module are expected to be sorted in order
-- 	of increasing centroid phi AND must be contained in the home sector and 
--  there must be no gaps in the clusters (L2_CLUSTSORT takes care of this).

-- 	the relatively static stuff like pm, oct, sect, turn, etc. should be stable
-- 	before frame_sel is asserted.  these signals should be stable
-- 	the entire time that the CPS record is being generated.

-- 	frame_sel is generated by L2_CLUSTSORT as follows:
--	

--		frame_sel		output
--		---------		-----------------------
--		01				CPS L2 header frame
--		10				Turn and Crossing frame
--		11				CPS L2 data frame
--		others			zeros

-- *** revision history ***
--
--  8 Apr 2002: created, functionally simulated.
-- 24 Sep 2002: stripped down, CPS functionality only.
-- 16 Oct 2002: register the dynamic inputs. Note: tvalid and centroid are not used!
-- 22 Oct 2002: fix cluster centroid bug.
-- 23 Oct 2002: add support for fake L2CPS record.  This record has PM=10, OCT=00, Number
--				of objects=0, turn=0x1234, crossing=0x56.
-- 25 Apr 2003: better fake L2CPS record
-- 13 May 2003:  consistant curve 1 = CCW

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity L2CPS_format is
	port(
		reset:		in	std_logic;	  					-- active low, async.
		clock:		in	std_logic;						-- 53MHz
		send_fake:	in	std_logic;

		frame_sel:	in	std_logic_vector( 1 downto 0);	-- select the type of word to be output

		-- static L2 record info
		
		pm:			in	std_logic_vector( 1 downto 0);	-- pass and mark bits
		octant:		in	std_logic_vector( 2 downto 0);	-- octant number (0-7)
		sector:		in	std_logic_vector( 3 downto 0);	-- relative sector within an octant (0-9)
		turn:		in	std_logic_vector(15 downto 0);	-- turn counter
		cross:		in	std_logic_vector( 7 downto 0);	-- crossing counter
		iso:		in	std_logic;						-- a single isolated track

		cluster_count:	in	std_logic_vector(3 downto 0);	-- number of found clusters to be reported for L2CPS [0-8]

		-- raw tracks and cluster information from L2_CLUSTSORT module
			
		-- cvalid:		in	std_logic;						-- cluster data from the L2engine, removed 7MAY04
		centroid:	in	std_logic_vector(4 downto 0);
		width:		in	std_logic_vector(2 downto 0);

		-- tvalid:		in	std_logic;						-- track data from the L2engine
		curve:		in	std_logic;
		pt:			in	std_logic_vector(1 downto 0);
		extpt:		in 	std_logic_vector(2 downto 0);
		cm:			in	std_logic;

		dout:		out	std_logic_vector(27 downto 0)	-- record output, parity has NOT been addeded, yet.
	);
end L2CPS_format;

architecture L2CPS_format_arch of L2CPS_format is

	-- declare registers...

		signal dout_reg:		std_logic_vector(27 downto 0);
		signal centroid_reg:	std_logic_vector(4 downto 0);
		signal width_reg:		std_logic_vector(2 downto 0);
		signal frame_sel_reg:	std_logic_vector(1 downto 0);
		signal curve_reg:		std_logic;
		signal pt_reg:			std_logic_vector(1 downto 0);
		signal extpt_reg:		std_logic_vector(2 downto 0);
		signal cm_reg:			std_logic;
		signal fake_count_reg:	std_logic_vector(2 downto 0);

	-- declare signals...

		signal cent:		std_logic_vector(4 downto 0);

		signal frame_mux:	std_logic_vector(27 downto 0);	
		signal turn_frame:	std_logic_vector(27 downto 0);

		signal cps_ob_hp:	std_logic_vector( 1 downto 0);
		signal cps_ob_vp:	std_logic_vector( 2 downto 0);
		
		signal cps_header_frame:  std_logic_vector(27 downto 0);

		signal cps_data_frame:  std_logic_vector(27 downto 0);
		
		signal r_bit: std_logic;

		signal fake_header:	std_logic_vector(27 downto 0);
		-- fake turn frame is the real turn frame...
		signal fake_clust1:	std_logic_vector(27 downto 0);
		signal fake_clust2:	std_logic_vector(27 downto 0);
		
	-- declare components...

begin

	-- combinatorial stuff...

			dout(27 downto 0)	<= dout_reg(27 downto 0);

		-- calculate the vertical and hortizonal EVEN parity fields for the
		-- CPS number of clusters.

			cps_ob_hp(1)	<= 	cluster_count(3);

			cps_ob_hp(0)	<= 	'1' when (cluster_count(2 downto 0)="001") else
								'1' when (cluster_count(2 downto 0)="010") else
								'1' when (cluster_count(2 downto 0)="100") else
								'1' when (cluster_count(2 downto 0)="010") else
								'1' when (cluster_count(2 downto 0)="111") else
								'0';

			cps_ob_vp(2)	<=	cluster_count(2);

			cps_ob_vp(1)	<=	cluster_count(1);

			cps_ob_vp(0)	<=	cluster_count(3) xor cluster_count(0);


		-- L2CPS header frame (horizontal parity bit will be modified later)

			cps_header_frame(27 downto 0)	<= 	"1110" & pm(1 downto 0) & octant(2 downto 0) & cps_ob_vp(2 downto 0) &
												"00" & cluster_count(3 downto 0) & cps_ob_hp(1 downto 0) &
												"11000" & "000";

		-- build up the fake L2CPS record...

			fake_header(27 downto 0) <= "111010" & octant(2 downto 0) & "0100000100111000000";
			fake_clust1(27 downto 0) <= "000000011001101000000" & sector(3 downto 0) & "000";
			fake_clust2(27 downto 0) <= "000001101011101100110" & sector(3 downto 0) & "000";

		-- the R bit is set if a cluster centroid is outside of the home sector.  for L2CPS this bit 
		-- will always be zero because the list of clusters provided to this module insures that the
		-- clusters are within the home sector ( 7 < centroid < 24).
		-- see engineering note 2002-01-22a for more information.

			r_bit <= '0';

		-- the reported centroid must be in the range of 0-15.  subtract off 8 to make it work.

			cent(4 downto 0) <= centroid_reg(4 downto 0) - "01000";


    	-- build the L2 CPS data frame from static variables (sector, etc.) and raw data coming from
		-- L2_CLUSTSORT.  (horizontal parity bit will be modified later)

			cps_data_frame(27 downto 0)	<= 	"00000" & cent(3 downto 0) & width_reg(2 downto 0) &
											cm_reg & '0' & curve_reg & pt_reg(1 downto 0) & extpt_reg(2 downto 0) &
											r_bit & sector(3 downto 0) & "00" & iso;

		-- turn and crossing frame (horizontal parity bit will be modified later)

			turn_frame(27 downto 0)	<= "0000" & turn(15 downto 0) & cross(7 downto 0);

		-- large mux to select which frame to output

			frame_mux(27 downto 0)	<=	cps_header_frame(27 downto 0) 	when (frame_sel_reg(1 downto 0)="01" and send_fake='0') else
										turn_frame(27 downto 0) 		when (frame_sel_reg(1 downto 0)="10" and send_fake='0') else
										cps_data_frame(27 downto 0) 	when (frame_sel_reg(1 downto 0)="11" and send_fake='0') else

										fake_header(27 downto 0) 		when (fake_count_reg(2 downto 0)="000" and send_fake='1') else
										turn_frame(27 downto 0) 		when (fake_count_reg(2 downto 0)="001" and send_fake='1') else
										fake_clust1(27 downto 0) 		when (fake_count_reg(2 downto 0)="010" and send_fake='1') else
										fake_clust2(27 downto 0) 		when (fake_count_reg(2 downto 0)="011" and send_fake='1') else
										X"0000000";

	-- sequential stuff...

--		register_stuff: process(clock, reset)	 change to sync reset
		register_stuff: process(clock)
			begin
			  if rising_edge(clock) then
			 	 if (reset='0') then
					dout_reg(27 downto 0)		<= X"0000000";
					centroid_reg(4 downto 0)	<= "00000";
					width_reg(2 downto 0)		<= "000";
					frame_sel_reg(1 downto 0) 	<= "00";
					curve_reg					<= '0';
					pt_reg(1 downto 0)			<= "00";
					extpt_reg(2 downto 0)		<= "000";
					cm_reg						<= '0';
					fake_count_reg(2 downto 0)	<= "111";
				 else
					dout_reg(27 downto 0)		<= frame_mux(27 downto 0);
					centroid_reg(4 downto 0)	<= centroid(4 downto 0);
					width_reg(2 downto 0)		<= width(2 downto 0);
					frame_sel_reg(1 downto 0) 	<= frame_sel(1 downto 0);
					curve_reg					<= curve;
					pt_reg(1 downto 0)			<= pt(1 downto 0);
					extpt_reg(2 downto 0)		<= extpt(2 downto 0);
					cm_reg						<= cm;

					if (send_fake='1' and frame_sel(1 downto 0)="01") then
						fake_count_reg(2 downto 0)	<= "000";
					elsif (fake_count_reg(2 downto 0) /= "111") then
						fake_count_reg(2 downto 0)	<= fake_count_reg(2 downto 0) + 1;
					end if;

				 end if;
			  end if;
		end process;


	-- instantiate components...

end L2CPS_format_arch;
